<cfcomponent output="false">

	<cffunction name="execute" access="public" output="false" returntype="boolean">
		<cfargument name="attributes" required="true"/>
		<cfset var local = structNew() />
		<cfset local.customerGateway = createObject("component","model.customerGateway") />
		<cfset local.result = "false" />
		<cfif isLoggedIn() AND hasChallengeQuestionSet( attributes, local.customerGateway )>
			<cfset local.result = "true" />
		<cfelseif challengeInfoSubmitted( arguments.attributes ) >
			<cfif isValidChallengeInfo( arguments.attributes ) >
				<cfset saveChallengeAnswer( arguments.attributes, local.customerGateway )>
				<cfset local.result = "true" />
			<cfelse>
				<cfset attributes.userMessage= "Please select a security question and provide an answer." />
			</cfif>	
		</cfif>		
		<cfreturn local.result />
	</cffunction>
	
	<cffunction name="isLoggedIn" output="false" access="private" returntype="boolean">
		<cfset var rtn = false />
		<cfif structKeyExists( client, "customerID" ) AND val( client.customerID ) GT 0 >
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />			
	</cffunction>
		
	<cffunction name="isValidChallengeInfo" output="false" access="private" returntype="boolean">
		<cfargument name="attributes" type="struct" required="true"/>
		<cfset var rtn = false />
		<cfif validateChallengeQuestion( arguments.attributes ) AND validateChallengeAnswer( arguments.attributes ) >
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />	
	</cffunction>
		
	<cffunction name="validateChallengeQuestion" output="false" access="private" returntype="boolean">
		<cfargument name="attributes" type="struct" required="true"/>
		<cfset var rtn = false />
		<cfif structKeyExists( arguments.attributes, "regChallengeQuestionID") AND val( attributes.regChallengeQuestionID ) GT 0 >
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />	
	</cffunction>
	
	<cffunction name="validateChallengeAnswer" output="false" access="private" returntype="boolean">
		<cfargument name="attributes" type="struct" required="true"/>
		<cfset var rtn = false />
		<cfif structKeyExists( arguments.attributes, "regChallengeAnswer") AND len( trim( attributes.regChallengeAnswer ) ) GT 0 >
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />
	</cffunction>
		
	<cffunction name="saveChallengeAnswer" output="false" access="public" returntype="void">
		<cfargument name="attributes" type="struct" required="true"/>	
		<cfargument name="customerGW" type="model.customerGateway" required="true"/>
			<cfset var cust =  arguments.customerGW.getById( client.customerID ) /> />
			<cfset cust.setChallengeQuestionID( attributes.regChallengeQuestionID ) />
			<cfset cust.setChallengeAnswer( attributes.regChallengeAnswer ) />
			<cfset arguments.customerGW.save( cust ) />
	</cffunction>
	
	<cffunction name="hasChallengeQuestionSet" output="false" access="private" returntype="boolean">
		<cfargument name="attributes" type="struct" required="true"/>
		<cfargument name="customerGW" type="model.customerGateway" required="true"/>
		<cfset var rtn = false />
		<cfif  val( arguments.customerGW.getByID( val(client.customerID ) ).getChallengeQuestionID() ) GT 0>
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />	
	</cffunction>

	
	<cffunction name="challengeInfoSubmitted" output="false" access="private" returntype="boolean">
		<cfargument name="attributes" type="struct" required="true"/>
		<cfset var rtn = false />
		<cfif structKeyExists( arguments.attributes, "regChallengeAnswer") AND structKeyExists( arguments.attributes, "regChallengeQuestionID") >
			<cfset rtn = true />
		</cfif>		
		<cfreturn rtn />	
	</cffunction>
	

</cfcomponent>